package com.agiledeveloper;

import java.io.File;
import org.neo4j.graphdb.factory.GraphDatabaseFactory;
import org.neo4j.graphdb.Transaction;
import org.neo4j.graphdb.RelationshipType;
import org.neo4j.graphdb.Relationship;
import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.GraphDatabaseService;
import org.neo4j.kernel.impl.util.FileUtils;
import org.neo4j.graphdb.index.Index;

public class Neo4JSample {
  private final static String DB_PATH = "./neo4j_db";
  private final static String PERSON = "person";
  private final static String NAME = "name";
  private final static String LANGUAGE = "language";
  private GraphDatabaseService neo4jdb;
  private Index<Node> nodeIndex;
  
  private static enum RelTypes implements RelationshipType {
    PROGRAMSIN,
    ISLEARNING
  }
  
  public void createDB() {
    neo4jdb = new GraphDatabaseFactory().newEmbeddedDatabase(DB_PATH);    
  }

  public Node createNode(String propertyName, String value) {
    Node node = neo4jdb.createNode();
    node.setProperty(propertyName, value);
    return node;
  }

  public void createData() {
    Transaction transaction = neo4jdb.beginTx();
    try {
      Node james = createNode(NAME, "James");
      Node sara = createNode(NAME, "Sara");
      Node bill = createNode(NAME, "Bill");
      Node java = createNode(LANGUAGE, "Java");
      Node groovy = createNode(LANGUAGE, "Groovy");
      Node scala = createNode(LANGUAGE, "Scala");
      
      james.createRelationshipTo(java, RelTypes.PROGRAMSIN);
      sara.createRelationshipTo(java, RelTypes.PROGRAMSIN);
      james.createRelationshipTo(groovy, RelTypes.ISLEARNING);
      sara.createRelationshipTo(scala, RelTypes.ISLEARNING);
      bill.createRelationshipTo(scala, RelTypes.ISLEARNING);
      
      nodeIndex = neo4jdb.index().forNodes("nodes");
      nodeIndex.add(james, PERSON, "JAMES");
      nodeIndex.add(sara, PERSON, "SARA");
      
      transaction.success();
    } finally {
      transaction.finish();
    }
  }
  
  public void printDetailsFor(String name) {
    System.out.println("-----------");
    Node person = nodeIndex.get(PERSON, name).getSingle();
    System.out.println("Person: " + person.getProperty(NAME));
    
    for(Relationship relationship : person.getRelationships()) {
      System.out.print(relationship.getType() + " " + relationship.getEndNode().getProperty(LANGUAGE) + " ");
      
      System.out.print("Everyone doing that:");
      for(Relationship relation : relationship.getOtherNode(person).getRelationships(Direction.INCOMING, relationship.getType())) {
        System.out.print(relation.getStartNode().getProperty(NAME) + " ");
      }
      
      System.out.println();
    }
  }
  
  public void useData() {
    printDetailsFor("JAMES");
    printDetailsFor("SARA");
  }
  
  public void shutdownDB() {
    neo4jdb.shutdown();
  }
  
  public static void main(String[] args) {
    Neo4JSample sample = new Neo4JSample();
    sample.createDB();
    sample.createData();
    sample.useData();
    sample.shutdownDB();
  }
}